Exception: CipherStash::Client::Error::InternalError

Inherits:
CipherStash::Client::Error show all
Defined in:
lib/cipherstash/client/error.rb

Overview

Something went wrong that indicates a bug in CipherStash::Client.

Reporting this exception as an issue is strongly encouraged. Please include the full error message, stacktrace, and the code that caused it, please.