Exception: CipherStash::Client::Error::QueryConstraintError

Inherits:
CipherStash::Client::Error show all
Defined in:
lib/cipherstash/client/error.rb

Overview

A query constraint was specified incorrectly.

Either a non-existent index was referenced, or an operator was specified that is not supported on the given index.