Class: CipherStash::Collection::QueryResult
- Inherits:
-
Object
- Object
- CipherStash::Collection::QueryResult
- Defined in:
- lib/cipherstash/collection/query_result.rb
Overview
The result set that comes from a query.
Instance Attribute Summary collapse
-
#aggregates ⇒ Array<CipherStash::Aggregate>
readonly
The aggregates that were derived from the query.
-
#records ⇒ Array<CipherStash::Record>
readonly
The set of records which was considered pertinent to the query.
Instance Attribute Details
#aggregates ⇒ Array<CipherStash::Aggregate> (readonly)
The aggregates that were derived from the query.
15 16 17 |
# File 'lib/cipherstash/collection/query_result.rb', line 15 def aggregates @aggregates end |
#records ⇒ Array<CipherStash::Record> (readonly)
The set of records which was considered pertinent to the query.
9 10 11 |
# File 'lib/cipherstash/collection/query_result.rb', line 9 def records @records end |