Class: CipherStash::AccessKey
- Inherits:
-
Object
- Object
- CipherStash::AccessKey
- Defined in:
- lib/cipherstash/access_key.rb
Overview
Represents a CipherStash access key.
An access key is a long-term credential suitable for use by non-interactive applications to authenticate their use of a CipherStash workspace.
Instance Attribute Summary collapse
-
#created_at ⇒ Time
readonly
When the access key was created.
-
#id ⇒ String
readonly
The globally-unique ID of the access key.
-
#last_used_at ⇒ Time, NilClass
readonly
When the access key was last exchanged for an access token.
-
#name ⇒ String
readonly
The name given to the access key when it was created.
-
#secret_key ⇒ String
readonly
The complete secret access key.
-
#workspace_id ⇒ String
readonly
The ID of the workspace that the access key grants access to.
Instance Attribute Details
#created_at ⇒ Time (readonly)
When the access key was created.
27 28 29 |
# File 'lib/cipherstash/access_key.rb', line 27 def created_at @created_at end |
#id ⇒ String (readonly)
The globally-unique ID of the access key.
17 18 19 |
# File 'lib/cipherstash/access_key.rb', line 17 def id @id end |
#last_used_at ⇒ Time, NilClass (readonly)
When the access key was last exchanged for an access token.
32 33 34 |
# File 'lib/cipherstash/access_key.rb', line 32 def last_used_at @last_used_at end |
#name ⇒ String (readonly)
The name given to the access key when it was created.
12 13 14 |
# File 'lib/cipherstash/access_key.rb', line 12 def name @name end |
#secret_key ⇒ String (readonly)
this method will only provide the access key for an object returned from CipherStash::Client#create_access_key. Listing access keys does not provide the secret access key.
The complete secret access key.
41 42 43 |
# File 'lib/cipherstash/access_key.rb', line 41 def secret_key @secret_key end |
#workspace_id ⇒ String (readonly)
The ID of the workspace that the access key grants access to.
22 23 24 |
# File 'lib/cipherstash/access_key.rb', line 22 def workspace_id @workspace_id end |